#include "ShaderConstants.fxh"

struct PS_Input
{
	float4 position : SV_Position;
	float4 color : COLOR;
	float3 ESBECloud : ESBECloud;
	float ESBESky : ESBESky;
};
struct PS_Output
{
	float4 color : SV_Target;
};

#ifdef RENDER_SKY

//=*=clouds=*=//
float hash(float2 p)
{
	p = frac(p * 0.1031);
	p += dot(p, p + 10.19);
	return frac((p.x + p.y) * p.y);
}
float noise( float2 p )
{
	float2 i = floor( p );
	float2 f = frac( p );
	float2 u = f*f*(4.0-3.0*f);
	return lerp( lerp( hash( i + float2(0.0,0.0) ), hash( i + float2(1.0,0.0) ), u.x),lerp( hash( i + float2(0.0,1.0) ), hash( i + float2(1.0,1.0) ), u.x), u.y);
}
float fbm(float2 p){
	return noise(p) * 0.6 + noise(p*3.0) * 0.300 + noise(p*6.0) * 0.21 + noise(p*4.0) * 0.064 + noise(p*44.0) * 0.032 + noise(p*48.0) * 0.032;
}
float cloudH2(float2 p, float mm) {
	float m = 4.0;
	float2 q = float2(fbm(float2(p)), fbm(p+float2(5.12*TIME*0.01, 1.08)));
	float2 r = float2(fbm((p+q*m)+float2(0.1, 0.741)), fbm((p+q*m)+float2(0.952, 0.845)));
	m /= mm;
	return fbm(p+r*m);
}
//=*=end clouds=*=//

//=*=aurora=*=//
float tri(float x){return clamp(abs(frac(x)-.5),0.001,0.52);}
float triNoise2d(float2 p)
{
    float z=1.8;
    float z2=2.5;
    float rz = 0.;
    p *= 0.55*(p.x*0.2);
    float2 bp = p;
	for (float i=0.; i<5.; i++ )
	{
		p -= (bp*1.85)*4.5/z2;
        bp *= 1.3;
        z2 *= .45;
        z *= .42;
        p *= 1.21 + (rz-1.0)*.02;
        rz += tri(p.x+tri(p.y))*z;
        p*= -1.0;
	}
    return clamp(1./pow(rz*29., 1.3),0.,.55);
}
float4 aurora(float3 rd)
{
    float4 col = 0.0;
		float of = 0.006*frac(sin(0.96));
		float b = rd.y*2.+0.4;
    float3 c = sin(TIME*0.01)*0.5+1.5-float3(2.15,-.5, 1.2);
    for(float i=0.;i<16.;i++){
        float pt = (.8+pow(i+1.,2.0)*0.002)/b;
        float3 bpos = float3(0.0,0.0,-6.5) + (pt-of)*rd;
        float rzt = triNoise2d(bpos.zx);
        float4 col2 = float4(float3((sin(c+i*0.1)*0.5+0.5)*rzt),rzt)*.5;
        col += col2*exp2(-i*0.065 - 2.5)*smoothstep(0.,5., i);
    }
	return col*(clamp(rd.y*15.+.4,0.,1.))*3.;
}
//=*=end aurora=*=//

void main( in PS_Input PSInput, out PS_Output PSOutput )
{
	float4 _color = PSInput.color;

//=*=aurora=*=//
	float daylight = smoothstep(0.25,0.5,FOG_COLOR.r + FOG_COLOR.g + FOG_COLOR.b);
	float rain = smoothstep(0.8,1.0,FOG_CONTROL.y);
	if(daylight<1.0 && rain>0.0){
		float2 p = PSInput.ESBECloud.xz*4.0;
		float fade = smoothstep(0.,0.01,abs(normalize(float3(p.x,1.0,p.y)).y))*0.1+0.9;
		float4 aur = smoothstep(0.,1.5,aurora(normalize(float3(p.x,1.0,p.y))))*fade;
		_color.rgb = lerp( aur.rgb*rain , _color.rgb , daylight);
	}
//=*=end aurora=*=//

//=*=clouds=*=//
	float night = FOG_COLOR.r + FOG_COLOR.g + FOG_COLOR.b + (1.0-PSInput.ESBESky)*PSInput.ESBESky;
	float2 xz = ((PSInput.ESBECloud.xz)*7.6);
	xz.y += xz.y*2.0/500.0;
	xz.x += TIME*0.0125;
	float ES = cloudH2(xz, 16.0+fbm(xz)*2.0);
	float ES2 = cloudH2(xz+float2(0.1,0.1), 12.0+fbm(xz)*16.1);
	float4 clouds = lerp(float4(1.0,1.0,1.0,2.0)*0.8,float4(1.0,1.0,1.0,1.0), min(max(smoothstep(0.5, 1.0, ES2),0.0),1.0));
	float dusk = pow(min(1.0, abs((CURRENT_COLOR.b-0.4)*10.0)), 1.5);
	clouds.rgb *= float3(2.0-dusk,1.0,dusk);
	_color = lerp(_color,clouds*night*0.35,saturate(smoothstep(0.5, 1.0, ES)));
//=*=end clouds=*=//

	PSOutput.color = lerp(_color, FOG_COLOR,PSInput.ESBESky );
}



#else

ROOT_SIGNATURE
void main(in PS_Input PSInput, out PS_Output PSOutput){PSOutput.color = PSInput.color;}

#endif
